# Copyright (C) 2019 Majormode.  All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import re

from majormode.perseus.constant import regex


class Email:
    """
    Represent a message to be sent as an electronic mail to recipient(s).
    """
    # Regular expression that matches an email address.
    REGEX_EMAIL_ADDRESS = None  # Lazy evaluation

    def __init__(
            self,
            recipient_email_addresses,
            author_name,
            author_email_address,
            subject,
            content):
        """
        Build an instance `Email`.


        :param recipient_email_addresses: email address(es) of the primary
            recipient(s) of the message.  A bare string will be treated as a
            list with one address.

        :param author_name: name of the person of system responsible for the
            writing of the message.

        :param author_email_address: email address of the mailbox to which the
            author of the message suggests that replies be sent.

        :param subject: a short string identifying the topic of the message.

        :param content: the body of the message.
        """
        # Cleanse email addresses provided to this constructor.
        recipient_email_addresses = [recipient_email_addresses.strip().lower()] if isinstance(recipient_email_addresses, str) \
            else [email_address.strip().lower() for email_address in recipient_email_addresses]

        author_email_address = author_email_address.strip().lower()

        # Check that the email addresses provided to this construction are all
        # compliant with RFC 2822.
        for email_address in recipient_email_addresses:
            self.assert_valid_address(email_address)

        self.assert_valid_address(author_email_address)

        self.__recipient_email_addresses = recipient_email_addresses
        self.__author_name = author_name
        self.__author_email_address = author_email_address
        self.__subject = subject
        self.__content = content

    @classmethod
    def assert_valid_address(cls, address):
        """
        Raise an exception if the specified email address is not compliant
        with RFC 2822.


        :param address: an email address.


        @raise AssertError: if the specified email address is not compliant
            with RFC 2822.
        """
        assert cls.is_address_valid(address), 'The email address "%s" is not valid' % address

    @property
    def author_email_address(self):
        """
        Indicate the mailbox to which the author of the message suggests that
        replies be sent.


        :return: Email address of the mailbox to which the author of the
            message suggests that replies be sent.
        """
        return self.__author_email_address

    @property
    def author_name(self):
        """
        Return the name of the person of system responsible for the writing of
        the message.


        :return: the name of the person of system responsible for the  writing
            of the message.
        """
        return self.__author_name

    @property
    def content(self):
        """
        Return the body of the message.


        :return: the body of the message.
        """
        return self.__content

    @classmethod
    def is_address_valid(cls, address):
        """
        Indicate whether the specified email address is compliant with RFC
        2822.


        :param address: an email address.


        :return: `True` if the specified email address is compliant with RFC
            2822; `False` otherwise.
        """
        if cls.REGEX_EMAIL_ADDRESS is None:
            cls.REGEX_EMAIL_ADDRESS = re.compile(regex.REGEX_PATTERN_EMAIL_ADDRESS)

        return cls.REGEX_EMAIL_ADDRESS.match(address) is not None

    @property
    def recipient_email_addresses(self):
        """
        Email addresses of the primary recipients of the message.


        :return: the email addresses of the primary recipients of the message.
        """
        return self.__recipient_email_addresses

    @property
    def subject(self):
        """
        Return the topic of the message.


        :return: a short string identifying the topic of the message.
        """
        return self.__subject
