# Copyright (C) 2019 Majormode.  All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import functools
import re
import sys
import unidecode

from majormode.perseus.constant.regex import REGEX_PATTERN_EMAIL_ADDRESS


REGEX_EMAIL_ADDRESS = re.compile(REGEX_PATTERN_EMAIL_ADDRESS)  # Lazy evaluation


def inicap_string(s, cleanse=False):
    """
    Convert the first letter of each word to capital letter without
    touching the rest of the word.


    @param s: a string.

    @param cleanse: `True` to remove any separator character from the
        string, such as comma; `False` to keeps any character but space.


    @return: a string for which the first letter of each word has been
        capitalized without modifying the case of the rest of the word.
    """
    return s and ' '.join([
        word[0].upper() + word[1:]
        for word in (s.split() if cleanse else s.split(' '))])


def is_email_address(s):
    """
    Check whether the specified string corresponds to an email address.


    @param s: a string that is expected to be an email address.


    @return: `True` if the string corresponds to an email address,
        `False` otherwise.
    """
    return (s and REGEX_EMAIL_ADDRESS.match(s.strip().lower())) is not None


def string_hash_code(s):
    """
    Calculate the hash code of a string.


    :param s: a string.


    :return: the hash code of the string.
    """
    string_length = len(s)
    return functools.reduce(
        lambda x, y: x + y,
        [ord(s[i]) * 31 ** (string_length - i - 1) for i in range(string_length)]) % sys.maxsize


def strip_string_words(s):
    """
    Return the string with all its words stripped.


    :param s: a string.


    :return: the string with all its words stripped
    """
    return s and ' '.join([word for word in s.split(' ')])


def string_to_keywords(s, keyword_minimal_length=1):
    """
    Split the specified string in a list of ASCII keywords.

    Remove any punctuation character from the specified list of keywords,
    remove any double or more space character and represent Unicode
    characters in ASCII.


    @param s: a list of keywords strip out any punctuation characters.

    @param keyword_minimal_length: minimal number of characters of the
        keywords to be returned.


    @return: the set of keywords cleansed from any special Unicode
        accentuated character, punctuation character, and double space
        character.
    """
    # Convert the string to ASCII lower characters.
    ascii_string = unidecode.unidecode(s.strip()).lower()

    # Replace any punctuation character with space.
    punctuationless_string = re.sub(r'[.,\\/#!$%^&*;:{}=\-_`~()<>"\']', ' ', ascii_string)

    # Decompose the string into distinct keywords.
    keywords = set(punctuationless_string.split())

    # Filter out sub-keywords of less than 2 characters.
    return [keyword for keyword in keywords if len(keyword) > keyword_minimal_length]
